import os
import os.path
import threading
import sqlite3


def traversal_files(dir_path, signal):
    # 功能：以列表的形式分别返回指定路径下的文件和文件夹，不包含子目录
    # 参数：路径/信号/游标/功能编号
    # 返回值：路径下的文件夹列表 路径下的文件列表
    if not os.path.exists(dir_path):
        logger("ERROR", "clibs", f"数据文件夹{dir_path}不存在，请确认后重试......", "red", signal=signal)
    else:
        dirs, files = [], []
        for item in os.scandir(dir_path):
            if item.is_dir():
                dirs.append(item.path.replace("\\", "/"))
            elif item.is_file():
                files.append(item.path.replace("\\", "/"))

        return dirs, files


def init_logdb():
    global conn, cursor
    conn = sqlite3.connect(":memory:", isolation_level=None, check_same_thread=False, cached_statements=2048)
    cursor = conn.cursor()
    cursor.execute("PRAGMA journal_mode=wal")
    cursor.execute("PRAGMA wal_checkpoint=TRUNCATE")
    cursor.execute("PRAGMA synchronous=normal")
    cursor.execute("PRAGMA temp_store=memory")
    cursor.execute("PRAGMA mmap_size=30000000000")
    cursor.execute("PRAGMA cache_size=200000")
    cursor.execute(
        """
        create table if not exists logs(
            id integer primary key autoincrement,
            timestamp DATETIME DEFAULT(STRFTIME('%Y-%m-%d %H:%M:%f', 'NOW', 'localtime')),
            level text,
            module text,
            content text
        )
        """
    )


def db_lock(func):
    def wrapper(*args, **kwargs):
        try:
            lock.acquire(True)
            ret = func(*args, **kwargs)
        finally:
            lock.release()
        return ret

    return wrapper


@db_lock
def logger(level, module, content, color="black", flag="both", signal=""):
    global cursor
    if "move.monitor" in content:
        return

    if flag == "signal":
        signal.emit(content, color)
    elif flag == "cursor":
        cursor.execute(f"INSERT INTO logs (level, module, content) VALUES (?, ?, ?)", (level, module, content))
    elif flag == "both":
        signal.emit(content, color)
        cursor.execute(f"INSERT INTO logs (level, module, content) VALUES (?, ?, ?)", (level, module, content))


def handle_exception(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as err:
            print(f"{func.__name__} err = {err}")
            logger("DEBUG", "clibs", f"{func.__name__} err = {err}", flag="cursor")
    return wrapper


log_path = f"assets/logs"
lock = threading.Lock()
running = [0, 0, 0, 0, 0, 0, 0]  # 制动数据/转矩数据/激光数据/精度数据/制动自动化/转矩自动化/耐久数据采集
functions = ["制动数据处理", "转矩数据处理", "激光数据处理", "精度数据处理", "制动自动化测试", "转矩自动化测试", "耐久数据采集"]
levels = ["DEBUG", "INFO", "WARNING", "ERROR"]
ip_addr, ssh_port, socket_port, xService_port, external_port, modbus_port, upgrade_port = "", 22, 5050, 6666, 8080, 502, 4567
username, password = "luoshi", "luoshi2019"
INTERVAL, RADIAN, MAX_FRAME_SIZE, MAX_LOG_NUMBER = 1, 57.3, 1024, 10
c_md, c_hr, c_ec, c_pd, conn, cursor, search_records = None, None, None, None, None, None, None
status = {"mysql": 0, "hmi": 0, "md": 0, "ec": 0}
c_joint_vel, c_servo_trq, c_sensor_trq, c_estimate_trans_trq, c_safety_estop = 1, 2, 3, 4, 3  # 各个指标所在列

init_logdb()

# ============== ↓↓↓DEBUG CODE↓↓↓ ==============
for i in range(100):
    logger("DEBUG", "clibs", 'running123', flag="cursor")
    logger("INFO", "aio", 'running234', flag="cursor")
    logger("WARNING", "openapi", 'running345', flag="cursor")
    logger("ERROR", "brake", 'running456', flag="cursor")
